﻿using System;
using UnityEngine;
using System.Collections;

namespace djsoapyknuckles
{
    public class LightningController : MVRScript
    { 
        #region VARIABLES  
        protected JSONStorableFloat lightningIntensity;
        protected JSONStorableFloat timeBetweenLightningFlashLow;
        protected JSONStorableFloat timeBetweenLightningFlashHigh;
        protected JSONStorableFloat flashSpeedLow;
        protected JSONStorableFloat flashSpeedHigh;
        protected JSONStorableFloat flashLengthLow;
        protected JSONStorableFloat flashLengthHigh;
        protected JSONStorableFloat fadeAmoutAtEndofFlash;
        protected JSONStorableBool startOnLoad;
        private Light _light;
        #endregion

        #region INIT
        public override void Init()
        {
            try
            {
                StartCoroutine(InitDeferred());
                

                #region UI
                startOnLoad = new JSONStorableBool("Start on Scene Load", false, Should_Start_On_Load);
                RegisterBool(startOnLoad);
                CreateToggle(startOnLoad, true);

                lightningIntensity = new JSONStorableFloat("Lightning Intensity Level", 1.0f, 0.01f, 8.0f);
                RegisterFloat(lightningIntensity);
                CreateSlider(lightningIntensity, false);

                timeBetweenLightningFlashLow = new JSONStorableFloat("Time between lightning flashes low bound", 4.0f, 1.0f, 30.0f);
                RegisterFloat(timeBetweenLightningFlashLow);
                CreateSlider(timeBetweenLightningFlashLow, false);

                timeBetweenLightningFlashHigh = new JSONStorableFloat("Time between lightning flashes high bound", 10.0f, 1.0f, 60.0f);
                RegisterFloat(timeBetweenLightningFlashHigh);
                CreateSlider(timeBetweenLightningFlashHigh, false);

                flashSpeedLow = new JSONStorableFloat("Flash Speed low bound", 0.03f, 0.01f, 0.1f);
                RegisterFloat(flashSpeedLow);
                CreateSlider(flashSpeedLow, false);

                flashSpeedHigh = new JSONStorableFloat("Flash Speed high bound", 0.09f, 0.01f, 0.1f);
                RegisterFloat(flashSpeedHigh);
                CreateSlider(flashSpeedHigh, false);

                flashLengthLow = new JSONStorableFloat("Length of Lighting Flash low bound", 0.2f, 0.1f, 1.0f);
                RegisterFloat(flashLengthLow);
                CreateSlider(flashLengthLow, false);

                flashLengthHigh = new JSONStorableFloat("Length of Lighting Flash high bound", 0.6f, 0.1f, 1.0f);
                RegisterFloat(flashLengthHigh);
                CreateSlider(flashLengthHigh, false);

                fadeAmoutAtEndofFlash = new JSONStorableFloat("Fade out at End of Flash", 0.008f, 0.008f, 0.08f);
                RegisterFloat(fadeAmoutAtEndofFlash);
                CreateSlider(fadeAmoutAtEndofFlash, false);

                UIDynamicButton udb1 = CreateButton("Start Lightning", true);
                if (udb1 != null)
                {
                    udb1.button.onClick.AddListener(() => StartLightning());
                }
                UIDynamicButton udb2 = CreateButton("Stop Lightning", true);
                if (udb2 != null)
                {
                    udb2.button.onClick.AddListener(() => StopLightning());
                }
                #endregion

                #region HELP TEXT
                JSONStorableString helpText = new JSONStorableString("Help",
                    "       <color=#000><size=35><b>Lightning Controller</b></size></color>\n\n" +
                    "<color=#333>" +
                    "<b>Intensity Level:</b> sets the maximum brightness of flash between 0.01 and 8.0. " +
                    "Intensity will be randomized between 0.01 and the value you select.\n\n" +
                    "<b>Time between lightning flashes:</b> (in seconds) Lightning will flash at a random number of seconds between the low and high bounds.\n\n" +
                    "<b>Flash Speed:</b> the strobe speed of a single flash of lightning. This value is randomized between low bound and high bound per flash. \n\n"+
                    "<b>Length of Lightning Flash:</b> how long a single flash of lighting may last. This value is randomized between low bound and high bound per flash.\n\n"+
                    "<b>Fade out at End of Flash:</b> lightning will fade to black from the last intensity value of the flash incremented by this value. (lower value = slower fade)\n" +
                    "<i>Hint: set a value of 0.01 for a slower fade with Lightning Intensity Levels of less than 1.0.</i>\n\n"+
                    "</color>"
                );
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;
            }
            #endregion
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        #endregion

        #region CALLBACKS
        protected void Should_Start_On_Load(JSONStorableBool b)
        {
            if (startOnLoad.val)
            {
                StartCoroutine("TimeBetweenFlashes");
            }
        }
        protected void StartLightning()
        {
            StartCoroutine("TimeBetweenFlashes");
        }

        protected void StopLightning()
        {
            StopAllCoroutines();
        }
        #endregion

        #region COROUTINES

        IEnumerator RandomFlashTimer()
        {
            if (!SuperController.singleton.freezeAnimation)
            {
                float timeleft = 0;
                float lengthofFlash = UnityEngine.Random.Range(flashLengthLow.val, flashLengthHigh.val);
                //SuperController.LogMessage("length of flash: " + lengthofFlash);
                while (timeleft <= lengthofFlash)
                {
                    float wait_time = UnityEngine.Random.Range(flashSpeedLow.val, flashSpeedHigh.val);
                    float flashintensity = UnityEngine.Random.Range(0.01f, lightningIntensity.val);
                    _light.intensity = flashintensity;
                    timeleft += Time.deltaTime;
                    // SuperController.LogMessage("time left: " + timeleft);
                    yield return new WaitForSeconds(wait_time);
                }
                yield return StartCoroutine(Fade());
            }
        }
        protected IEnumerator TimeBetweenFlashes()
        {
            while (true)
            {

                float wait_time = UnityEngine.Random.Range(timeBetweenLightningFlashLow.val, timeBetweenLightningFlashHigh.val);
                //SuperController.LogMessage("low value: " + tbf_low);
                StartCoroutine("RandomFlashTimer");
                yield return new WaitForSeconds(wait_time);
                //SuperController.LogMessage("Length between flashes was: " + wait_time);
            }
        }
        protected IEnumerator Fade()
        {
            for (float ft = _light.intensity; ft >= 0.01; ft -= fadeAmoutAtEndofFlash.val)
            {
                _light.intensity = ft;
                //SuperController.LogMessage("fade time " + ft);
                yield return null;
            }
        }
        protected IEnumerator InitDeferred()
        {
            yield return new WaitForEndOfFrame();
            //SuperController.LogMessage("in initDefered");
            yield return new WaitForEndOfFrame();
            while (SuperController.singleton.isLoading)
            {
                //SuperController.LogMessage("is loading...");
                yield return null;
            }
            _light = containingAtom.reParentObject.GetComponentInChildren<Light>();
            if (_light == null)
            { throw new Exception("No Dummy, It's a Lightning plugin! This Plugin must be added to a LIGHT"); }
            Should_Start_On_Load(startOnLoad);
        }
    }
    #endregion
}


